import argparse
import logging
import sys
import time

from helper import *

REDIS_POD_NAME = 'redis-cluster-'
REDIS_PVC_NAME = 'redis-data-redis-cluster-'
REDIS_POD_0_NAME = REDIS_POD_NAME + '0'
FAILED_CLUSTER_STATE = 'cluster_state:fail'
REDIS_CONTAINER_STARTED = 'Started container redis-cluster'
RESTART_LIMIT = 6
REDIS_JOB_LABEL = "app.kubernetes.io/name=redis-cluster"
REDIS_JOB_NAME = "redis-startup-probe-initial"
REDIS_NAMESPACE = "nsxi-platform"
logging.basicConfig(format=FORMAT, datefmt=DATEFMT, level=logging.DEBUG)

def wait_for_redis_container():
    total_attempts = 20
    attempts = 0
    while attempts < total_attempts:
        attempts += 1
        response = get_statefulset_status()
        if response[0] != response[1]:
            logging.info("Replica count and ready replica count do not match. The expected count is {expected} and the running replica is {actual}".format(expected=response[1], actual=response[0]))
            if not check_redis_cluster_state():
                reset_redis_cluster()
                return
            else:
                logging.info("Redis not in cluster fail state, but waiting for all the pods to be in ready state")
                time.sleep(30)
        else:
            return
    reset_redis_cluster()
    return

def check_redis_cluster_state():
    while True:
        output = get_pod_cluster_status(REDIS_POD_0_NAME, FAILED_CLUSTER_STATE)
        if output is None:
            logging.error("Failed to check cluster state")
            sys.exit(1)
        if FAILED_CLUSTER_STATE in output:
            logging.info('Redis Cluster in failed state, attempting restart of pods')
            break
        else:
            logging.info('Redis Cluster not in failed state')
            return True
    return False

def check_redis_cluster_pod_exists():
    restart_count = 0
    while True:
        pods_list = get_list_of_redis_cluster_pods()
        if REDIS_POD_0_NAME not in pods_list:
            logging.error('Could not find pod {pod}'.format(pod=REDIS_POD_0_NAME))
            restart_count += 1
            if restart_count > RESTART_LIMIT:
                logging.error('Retry limit reached, exiting')
                return False
            time.sleep(10)
            continue
        else:
            logging.info('Found pod {pod}'.format(pod=REDIS_POD_0_NAME))
            break

    return True

def reset_redis_cluster():
    restart_count = 0
    redis_cluster_pods = get_list_of_redis_cluster_pods()
    redis_cluster_pvcs = get_list_of_redis_cluster_pvcs()
    while True:
        try:
            delete_pvc(redis_cluster_pvcs)
        except Exception as e:
            logging.error('Could not delete redis pvc')
            logging.error(e)
            restart_count += 1
            if restart_count > RESTART_LIMIT:
                logging.error('Retry limit reached, exiting')
                return False
            time.sleep(10)
            continue
        logging.info("Redis Cluster pvcs deleted")

        try:
            delete_pod(redis_cluster_pods)
        except Exception as e:
            logging.error('Could not delete redis pods')
            logging.error(e)
            restart_count += 1
            if restart_count > RESTART_LIMIT:
                logging.error('Retry limit reached, exiting')
                return False
            time.sleep(10)
            continue

        logging.info("Redis Cluster pods deleted")
        time.sleep(200) # need time for the pods to come back up and form cluster
        break
    return True

def main():
    parser = argparse.ArgumentParser(description='Redis job to help startup redis cluster.')
    parser.add_argument('--not-initial', '-i', action="store_true", help='Whether job is at startup or cronjob')
    args = parser.parse_args()

    if args.not_initial:
        logging.info("Checking whether initial startup job exists")
        if is_job_running(REDIS_JOB_NAME, REDIS_JOB_LABEL, REDIS_NAMESPACE):
            logging.info("Initial redis startup probe job is already running, exiting")
            return

    try_count = 0
    logging.info('Checking redis cluster state')

    exit_code = check_kubectl_command_connection()
    if exit_code > 0:
        logging.error('Could not connect to kubernetes cluster from redis startup job')
        return

    wait_for_redis_container()
    wait_for_redis_container()

    success = check_redis_cluster_state()
    while not success and try_count < RESTART_LIMIT:
        try_count += 1
        logging.info('Searching for redis-cluster pod')
        status = check_redis_cluster_pod_exists()
        if not status:
            logging.info('Failed to find redis cluster pod, retrying')
            continue

        logging.info('Starting reset of redis cluster')
        status = reset_redis_cluster()
        if not status:
            logging.info('Failed to restart redis cluster pods and pvcs, retrying')
            continue

        logging.info('Successfully reset redis cluster')
        logging.info('Rechecking redis cluster state')
        success = check_redis_cluster_state()

    if not success and try_count >= RESTART_LIMIT:
        logging.error('Failed to start redis cluster, retry limit reached')
    else:
        logging.info('Successfully started redis cluster')

if __name__ == '__main__':
    main()