import logging
import subprocess

from kubernetes import client, config
from kubernetes.client.rest import ApiException

FORMAT = '%(asctime)s %(levelname)s: %(message)s'
DATEFMT = '%Y-%m-%d %H:%M:%S'

logging.basicConfig(format=FORMAT, datefmt=DATEFMT, level=logging.DEBUG)

# execute command on pod via kubectl exec -it
# if execution is not correctly executed, will return None
def execute_command_for_pod(pod, command):
    command = "kubectl exec -i {pod} -c redis-cluster -- {command}".format(pod=pod, command=command)
    try:
        output = subprocess.check_output(command, shell=True)
        return output
    except Exception as e:
        logging.error("execution error for command: %s" %(command))
        logging.error(e)
        return None

def get_list_of_redis_cluster_pods():
    command = "kubectl get pods | grep redis-cluster-"
    redis_pods = []

    try:
        raw_output = subprocess.check_output(command, shell=True)
        pods = raw_output.decode().splitlines()
        for pod in pods:
            redis_pods.append(pod.split()[0])
    except:
        logging.info("execution error for command: %s" %(command))

    return redis_pods

def get_list_of_redis_cluster_pvcs():
    command = "kubectl get pvc | grep redis-data-redis-cluster-"
    redis_pvcs = []

    try:
        raw_output = subprocess.check_output(command, shell=True)
        pvcs = raw_output.decode().splitlines()
        for pvc in pvcs:
            redis_pvcs.append(pvc.split()[0])
    except:
        logging.info("execution error for command: %s" %(command))

    return redis_pvcs

def delete_pvc(names_list):
    names = ' '.join(names_list)
    command = "kubectl delete pvc {pvc}".format(pvc=names)

    # deleting pvc will hang indefinitely because pod still exists, so call without waiting for command completion
    subprocess.Popen(command, shell=True, stdin=None, stdout=None, stderr=None, close_fds=True)

def delete_pod(names_list):
    names = ' '.join(names_list)
    command = "kubectl delete pod {pod} --grace-period=0 --force".format(pod=names)

    subprocess.check_output(command, shell=True)

def get_pod_cluster_status(name, search_string):
    describe_command = "kubectl describe pod {pod}".format(pod=name)
    command = "kubectl describe pod {pod} | grep '{grep_string}'".format(pod=name, grep_string=search_string)

    try:
        output = subprocess.check_output(describe_command, shell=True)
        logging.info("Redis-custer-0 describe pod: {output}".format(output=output))
        grep_output = subprocess.check_output(command, shell=True)
        return grep_output.decode()
    except Exception as e:
        if e.returncode == 1:
            # this means grep did not find match, not actually an error
            return ""
        else:
            logging.error("execution error for command: %s" %(command))
            logging.error(e)
            return None

def get_statefulset_status():
    all_counts = []
    command_for_ready_replicas = "kubectl -n nsxi-platform get sts redis-cluster -o jsonpath={.status.readyReplicas}"
    command_for_replicas = "kubectl -n nsxi-platform get sts redis-cluster -o jsonpath={.status.replicas}"

    try:
        output_for_ready_replicas = subprocess.check_output(command_for_ready_replicas, shell=True)
        output_for_replicas = subprocess.check_output(command_for_replicas, shell=True)
        return [output_for_ready_replicas, output_for_replicas]
    except Exception as e:
        logging.error("execution error for kubectl commands")
        logging.error(e)
        return None

def is_job_running(job_name, label, namespace):
    try:
        config.load_incluster_config()
        batch_v1 = client.BatchV1Api()

        response = batch_v1.read_namespaced_job_status(namespace=namespace, name=job_name)
        return response.status.active == 1
    except ApiException as e:
        logging.info("Redis probe job doesn't exist")
        return False

# check that kubectl commands are working
def check_kubectl_command_connection():
    command = "kubectl get pods"

    try:
        subprocess.check_output(command, shell=True)
        return 0
    except Exception as e:
        logging.error("Failed to run kubectl command with error %s" % (str(e)))
        return e.returncode